/*
   Copyright (C) 2006,2007,2008 Novell Inc

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/poll.h>
#include <sys/types.h>
#include <fcntl.h>
#include <QTreeWidget>
#include <QObject>
#include <QTimer>
#include <QMessageBox>
#include <QDir>
#include <KGlobal>
#include <KGenericFactory>
#include <k3process.h>
#include <KDebug>
#include <KLocale>
#include <KMessageBox>
#include <KConfig>
#include <kstandarddirs.h>

#include "packagekit_updater.h"
#include "constants.h"
#include "mainwindow.h"

K_EXPORT_COMPONENT_FACTORY( kupdateapplet_packagekit, KGenericFactory<PackageKitUpdater>( "kupdateapplet_packagekit" ) )

#define ASSERT_LISTS_VALID if ( !_patchList || !_packageList || !_driverList ) { kDebug() << "FATAL: some lists is not valid"; return; }

PackageKitUpdater::PackageKitUpdater( QObject *parent, const QStringList& ) 
	: Updater( parent ) 
	, _patchList (0)
	, _packageList (0)
	, _securityAvailable (false)
        , _driverSearch(false)
        , _get_packages_mutex(new QMutex(QMutex::Recursive))
{
	// For ease announce that we have been loaded.
	kDebug() << "PackageKit plugin loaded";
	kDebug() << "Initializing PackageKit backend";

	// Connect to PackageKit 
	pkConnector = new QPackageKitConnector(this);

	licenseDialog = new PkLicenseDialog(NULL);

	// Package information when asked (by getupdates)
        connect(pkConnector,
		SIGNAL(Package(const QString&, const QString&, const QString&)),
		this,
		SLOT(slotNewPackage(const QString&, const QString&, const QString&))
		);

	// Package description when asked (by getdescription)
	connect(pkConnector, 
		SIGNAL(Details(const QString&, const QString&, const QString&,
			 const QString&, const QString&, qulonglong)),
		this,
		SLOT(slotGotDetails(const QString&, const QString&, const QString&,
			const QString&, const QString&, qulonglong))
	);

        // transaction finished
	connect(pkConnector,
		SIGNAL(Finished(QPackageKitConnector::Exit::ExitEnum, uint)),
		this,
		SLOT(slotTransactionFinished(QPackageKitConnector::Exit::ExitEnum, uint))
	);

        // getUpdates transaction finished
	connect(pkConnector,
		SIGNAL(GetUpdatesFinished()),
		this,
		SLOT(slotGetUpdatesFinished())
	);

        // getUpdates transaction finished
	connect(pkConnector,
		SIGNAL(InstallFinished()),
		this,
		SLOT(slotInstallFinished())
	);

        // transaction without progress information
        connect(pkConnector,
		SIGNAL(NoProgressUpdates()),
		this,
		SLOT(slotNoProgressInfo())
	);

        // transaction with progress information
        connect(pkConnector,
		SIGNAL(ProgressChanged(uint, uint, uint, uint)),
		this,
		SLOT(slotUpdateProgress(uint, uint, uint, uint))
	);

        connect( pkConnector,
                 SIGNAL(StatusChanged(const QString &)),
                 this,
                 SLOT(slotStatusChanged(const QString &))
            );

        
        connect(pkConnector,
		SIGNAL(ErrorMessage(const QString&)),
		this,
		SLOT(slotError(const QString&))
	);

	// Eula required
	connect(pkConnector,
		SIGNAL(EulaRequired(const QString&, const QString&, const QString&,
				const QString&)),
		this,
		SLOT(slotShowEula(const QString&, const QString&, const QString&,
			const QString&))
	);


	connect(pkConnector,
		SIGNAL(RepoSignatureRequired(const QString&, const QString&,
        	        const QString&, const QString&, const QString&,
	                const QString&, const QString&, const QString&)),
		this,
		SLOT(slotRepoSignatureRequired(const QString&, const QString&,
        	        const QString&, const QString&, const QString&,
	                const QString&, const QString&, const QString&))
	);

	connect(pkConnector,
		SIGNAL(RepoDetail(const QString& , const QString&, bool)),
		this,
		SLOT(slotRepoDetail(const QString& , const QString&, bool))
	);

	connect(pkConnector,
		SIGNAL(UpdateDetail(const QString&, const QString&, const QString&, const QString&,
                const QString&, const QString&, const QString&, const QString&,
      		const QString&, const QString&, const QString&, const QString&)),
		this,
		SLOT(slotUpdateDetail(const QString&, const QString&, const QString&, const QString&,
                const QString&, const QString&, const QString&, const QString&,
      		const QString&, const QString&, const QString&, const QString&))
	);

	connect(pkConnector,
		SIGNAL(RequireRestart(const QString&, const QString&)),
		this,
		SLOT(slotRequireRestart(const QString&, const QString&))
	);

	connect(pkConnector,
		SIGNAL(Message(const QString&, const QString&)),
		this,
		SLOT(slotMessage(const QString&, const QString&))
	);

	connect(pkConnector,
		SIGNAL(DistroUpgrade( const QString &, const QString &, const QString &)),
		this,
		SLOT(slotDistroUpgrade( const QString &, const QString &, const QString &))
	);

	connect(pkConnector,
		SIGNAL(RepoListFinished()),
		this,
		SLOT(slotUpdaterRepoSearchFinished())
	);


}

void PackageKitUpdater::slotStatusChanged( const QString &status )
{
	_lastStatus = status;
	if ( status == "finished" )
		emit progress( 100 , 100, status);
}


void PackageKitUpdater::slotTransactionFinished(QPackageKitConnector::Exit::ExitEnum exitCode, uint runtime)
{
	Q_UNUSED(runtime);
        
	kDebug() << "transaction finished (driver search: " << _driverSearch << ") exit code: " << exitCode << " runtime: " << runtime;
}


void PackageKitUpdater::slotError(const QString &message)
{
	// in case of an error without error message (e.g. eula needed)
	// we don't shown an error message
	if (message != "")
	{
		kDebug() << message;
		_errors = true;
		emit appletError(message);
	}
	else
	{
		kDebug() << "ignorring error";
		_softerrors = true;
	}
}

void PackageKitUpdater::slotNoProgressInfo()
{
	kDebug() << "noProgressInfo";
        emit progress( 0 , 0, "");
}


void PackageKitUpdater::slotUpdateProgress(uint percentage, uint subpercentage, uint elapsed, uint remaining) 
{
	Q_UNUSED(elapsed);
	Q_UNUSED(remaining);

	kDebug() << percentage << "(" << subpercentage << ")" ;
        emit progress( percentage , 100, _lastStatus);
}

// Called when PackageKit returns search results from GetUpdates command
void PackageKitUpdater::slotNewPackage(const QString &info, const QString &package_id, const QString &summary)
{
	// we don't want to add status messages about package installation
	// to our list of available packages
	if ( info == "installing" || info == "downloading" || info == "finished" )
		return;

        ASSERT_LISTS_VALID;
        
	// disassemble package_id
	QStringList id_list = package_id.split(";");
        QString name = id_list[0];
        QString version = id_list[1];
        QString arch = id_list[2];
        QString data = id_list[3];


	// add package to list
	QTreeWidgetItem *newItem;

	// map packagekit info to our type
	QString myInfo = info;


        if (myInfo == "important")
		myInfo = "recommended";
	else if (myInfo == "low")
		myInfo = "optional";
        else if (myInfo == "security")
		myInfo = "security";
	else if (myInfo == "enhancement")
		myInfo = "feature";
        else if (myInfo == "normal")
		myInfo = "normal";


	if ( _driverSearch )
        {
            kDebug() << "found package during driver search";
            newItem = new QTreeWidgetItem(_driverList);
        }
        else
        {
            kDebug() << "found package for updates";    
            newItem = new QTreeWidgetItem(_patchList);
        }
        
        newItem->setFlags(Qt::ItemIsUserCheckable 
                          | Qt::ItemIsEnabled
                          | Qt::ItemIsSelectable );

        newItem->setCheckState(0, Qt::Unchecked);
        newItem->setSelected(false);        

	newItem->setText(COLUMN_NAME, name );
	newItem->setText(COLUMN_SUMMARY, summary );
	newItem->setText(COLUMN_TYPE, myInfo );
	newItem->setText(COLUMN_NEW_VERSION, version );
	newItem->setText(COLUMN_CATALOG, data );
	newItem->setText(COLUMN_ID, package_id ) ;

	if (info == "security")
		_securityAvailable = true;

	
	// ask for package details because we want to know whether the restart flag is set
	//pkConnector->getUpdateDetail(QStringList(package_id));
	_packageDetailsList << package_id;
}

void PackageKitUpdater::slotGetUpdatesFinished()
{
	ASSERT_LISTS_VALID;

	kDebug() << "updates Finished (patches): "  <<  _patchList->children().count();
	kDebug() << "updates Finished (packages): " <<  _packageList->children().count();
	kDebug() << "updates Finished (drivers): "  <<  _driverList->children().count();

        _missing_details = _packageDetailsList.count();
	pkConnector->getUpdateDetail(QStringList( _packageDetailsList ));
 
	if (_errors || _softerrors )
	{
            kDebug() << "there were errors";
	    return;
	}

	slotWaitForDetailsComplete();
}


void PackageKitUpdater::slotWaitForDetailsComplete()
{
        if ( _missing_details == 0)
	{
        	kDebug() << "emiting check finished";
		emit checkFinished();
	}
	else
 		QTimer::singleShot(500, this, SLOT(slotWaitForDetailsComplete()));

}

void PackageKitUpdater::slotInstallFinished()
{
	if (_errors || _softerrors)
	{
            kDebug() << "there were errors";
	    return;
	}
	else
	{
	   doCheckForUpdates();
	}
}


void PackageKitUpdater::slotUpdaterRepoSearchFinished()
{
        kDebug() << "finished search for update repositories";
	if (!_updateRepoAvailable)
        {
            kDebug() << "no update repositories defined";
            emit pluginNeedsConfiguration( PLUGIN_CONFIG_NO_UPDATE_REPOS );
	}
	else
	{
	    doCheckForUpdates();
	}
}


void PackageKitUpdater::slotDriverSearchFinished()
{
        ASSERT_LISTS_VALID;
        
	kDebug() << "driver search finished (patches): "  <<  _patchList->children().count();
	kDebug() << "driver search finished (packages): " <<  _packageList->children().count();
	kDebug() << "driver search finished (drivers): "  <<  _driverList->children().count();
	if (_errors || _softerrors)
	{
		return;
	}

        emit checkFinished();
}


Updater::Capabilities PackageKitUpdater::capabilities() const
{
	return Updater::CanSelectIndividualUpdates;
}

void PackageKitUpdater::slotGotDetails(const QString &package_id, const QString &licence, const QString &group,
                        const QString &detail, const QString &url, qulonglong size) 
{
	Q_UNUSED(package_id);
	Q_UNUSED(licence);
	Q_UNUSED(group);
	Q_UNUSED(url);
	Q_UNUSED(size);
	emit returnDescription(detail, 0);	// ugly hack: set description of package and pathes
	emit returnDescription(detail, 1);	// as we don't know which resolvable type was queried
}



void PackageKitUpdater::slotUpdateDetail(const QString &package_id, const QString &updates, const QString &obsoletes, const QString &vendor_url,
                const QString &bugzilla_url, const QString &cve_url, const QString &restart, const QString &update_text,
      		const QString &changelog, const QString &state, const QString &issued, const QString &updated )
{

	Q_UNUSED(changelog);
	Q_UNUSED(state);
	Q_UNUSED(issued);
	Q_UNUSED(updated);

	kDebug() << "package_id:" << package_id << endl;
	kDebug() << "updates:" << updates << endl;
	kDebug() << "obsoletes:" << obsoletes << endl;
	kDebug() << "vendor_url:" << vendor_url << endl;
	kDebug() << "bugzilla_url:" << bugzilla_url << endl;
	kDebug() << "cve_url:" << cve_url << endl;
	kDebug() << "restart:" << restart << endl;
	kDebug() << "update_text:" << update_text << endl;

        ASSERT_LISTS_VALID;

	QTreeWidgetItem *foundItem = NULL; 
	{
            QTreeWidgetItemIterator it(_driverList);
            while (*it)
            {
                QTreeWidgetItem *item(*it);
                if ( item->text(COLUMN_ID) == package_id )
                    foundItem = item;
                ++it;
            }
        }
        // in packages
        {
            
            QTreeWidgetItemIterator it(_packageList);
            while (*it)
            {
                QTreeWidgetItem *item(*it);
                if ( item->text(COLUMN_ID) == package_id )
                    foundItem = item;
                ++it;
            }
        }
        // in patches
        {
            QTreeWidgetItemIterator it(_patchList);
            while (*it)
            {
                QTreeWidgetItem *item(*it);
                if ( item->text(COLUMN_ID) == package_id )
                    foundItem = item;
                ++it;
            }

        }
        
	if (foundItem)
	{
		foundItem->setText(COLUMN_RESTART_HIDDEN, restart );
		foundItem->setText(COLUMN_RESTART, restart );
	}

	_missing_details -- ;

}

void PackageKitUpdater::shutdown()
{

	kDebug() << "trying to stop PackageKit deamon...";
	pkConnector->suggestDaemonQuit();

	delete pkConnector;
}

void PackageKitUpdater::slotShowLog()
{
}


void PackageKitUpdater::doCheckForUpdateRepos()
{
        kDebug() << "try to get repolist";
	_updateRepoAvailable=false;
	pkConnector->getRepoList();
}


void PackageKitUpdater::doCheckForUpdates()
{
	_get_packages_mutex->lock();
	_driverSearch = false;

	_packageDetailsList.clear();


// NOT NEEDED ANYMORE
//        // first of all: check whether registration ran
//        if (!checkRegistrationRan())
//        {
//                kDebug() << "Registration didn't run.";
//		emit pluginNeedsConfiguration( PLUGIN_CONFIG_NO_UPDATE_REPOS );
//                return;
//        }

        kDebug() << "Checking for updates...";

	ASSERT_LISTS_VALID;

	_patchList->clear();
	_packageList->clear();
	_driverList->clear();
	_securityAvailable=false;
	_errors=false;
	_softerrors=false;

	pkConnector->getUpdates();
        
        _get_packages_mutex->unlock();
}

void PackageKitUpdater::slotCheckDrivers(QTreeWidget *patchList, QTreeWidget *packageList, QTreeWidget *driverList)
{
	_driverList = driverList;
	_patchList = patchList;
	_packageList = packageList;

	kDebug() << "got check driver request...";

	doCheckForDrivers();
}

void PackageKitUpdater::slotCheckFirmware(QTreeWidget *patchList, QTreeWidget *packageList, QTreeWidget *driverList)
{    
	_driverList = driverList;
	_patchList = patchList;
	_packageList = packageList;

	kDebug() << "got check firmware request...";

	doCheckForFirmware();
}


void PackageKitUpdater::doCheckForDrivers()
{
	_get_packages_mutex->lock();

	kDebug() << "Checking for drivers...";

        ASSERT_LISTS_VALID;
        
	_driverList->clear();
	_securityAvailable=false;
	_errors=false;
	_softerrors=false;

        _driverSearch = true;
	pkConnector->getDrivers();
        _get_packages_mutex->unlock();

}


void PackageKitUpdater::slotPopulateLists(QTreeWidget *patchList, QTreeWidget *packageList, QTreeWidget *driverList)
{
	_patchList = patchList;
	_packageList = packageList;
	_driverList = driverList;

	kDebug() << "got populate UI request...";
	
	doCheckForUpdateRepos();
	
	// check whether update repos exist.
	// CheckForUpdates() is called if update repos are available
}

void PackageKitUpdater::slotResolvableSelected(QTreeWidgetItem *item, int resolvableType)
{
	Q_UNUSED(resolvableType);
	kDebug() << "retrieve updating item";
	
	if (item)
	{
		QString id = item->text(COLUMN_ID);
		kDebug() << "getDetails" <<  id;
		pkConnector->getDetails(QStringList(id));
	}
}


void PackageKitUpdater::slotStartInstall()
{
	kDebug() << "start installation...";
	_errors = false;
	_softerrors = false;

        ASSERT_LISTS_VALID;
        
        emit progress( 0 , 1, "");

	QStringList packageIds;

	// iterate through list of updates the user wants to install and trigger installation

        {
            QTreeWidgetItemIterator it(_packageList);
            while (*it)
            {
                QTreeWidgetItem *item(*it);
                if ( (item->checkState(0) == Qt::Checked) 
                     && item->text(COLUMN_NAME) != "dummypatchforappletstate" )
                {
                    packageIds << item->text(COLUMN_ID);
                }		

                ++it;
            }
        }

        {
            QTreeWidgetItemIterator it(_patchList);
            while (*it)
            {
                QTreeWidgetItem *item(*it);
                if ( (item->checkState(0) == Qt::Checked) 
                     && item->text(COLUMN_NAME) != "dummypatchforappletstate" )
                {
                    packageIds << item->text(COLUMN_ID);
                }		

                ++it;
            }
        }

        {
            QTreeWidgetItemIterator it(_driverList);
            while (*it)
            {
                QTreeWidgetItem *item(*it);
                if ( (item->checkState(0) == Qt::Checked) 
                     && item->text(COLUMN_NAME) != "dummypatchforappletstate" )
                {
                    packageIds << item->text(COLUMN_ID);
                }		

                ++it;
            }
        }

	pkConnector->updatePackages(packageIds);

}
	
void PackageKitUpdater::slotRepoSignatureRequired(const QString &package_id, const QString &repository_name,
                const QString  &key_url, const QString &key_userid, const QString &key_id,
                const QString &key_fingerprint, const QString &key_timestamp, const QString &type)
{



	int answ =QMessageBox::question(NULL,i18n("Repository Signature Required"), 
		i18n("Do you want to accept this repository signature?") +
		"\npackage_id :" + package_id +
		"\nrepository_name: " +repository_name +
		"\nkey_url: " +  key_url +
		"\nkey_userid:" + key_userid +
		"\nkey_id: " + key_id +
		"\nkey_fingerprint: " + key_fingerprint +
		"\nkey_timestamp: " + key_timestamp +
		"\ntype: " + type, 
		QMessageBox::Yes | QMessageBox::No) ;
	
	if (answ == QMessageBox::Yes )
	{
		pkConnector->installSignature(type, key_id, package_id);
		_softerrors=false;
		pkConnector->triggerLastActionAgain();
	}
	else
	{
		//declined
		_softerrors=false;
		emit checkFinished();	// workaround
	}



}

void PackageKitUpdater::slotStopPackageManagementDaemon()
{
	kDebug() << "trying to stop PackageKit deamon...";
	pkConnector->suggestDaemonQuit();
}	

void PackageKitUpdater::slotConfigureUpdater()
{
	kDebug() << "configure updater";

	QProcess *inst_source = new QProcess(this);
	QStringList args;
	args << "-c" << "/sbin/yast2 inst_source";

	inst_source->start( KStandardDirs::findExe("kdesu"), args, QIODevice::NotOpen );

}

void PackageKitUpdater::slotShowEula(const QString& eula_id, const QString& package_id, const QString& vendor_name, const QString& license_agreement)
{
	Q_UNUSED(vendor_name);

	licenseDialog->setText(i18n("License of %1", package_id), license_agreement);
	if( licenseDialog->exec() )
	{
		pkConnector->acceptEula(eula_id);
		_softerrors=false;
		pkConnector->triggerLastActionAgain();
	}
	else
	{
		//declined
		doCheckForUpdates();
	}
}

bool PackageKitUpdater::checkRegistrationRan()
{
        QFile file("/var/lib/YaST2/registrationDidRun");
        if ( file.exists() )
        {
            kDebug() << "registration already ran";
            return true;
        }
        kDebug() << "registration did not run yet";
        return false;
}


void PackageKitUpdater::slotRepoDetail(const QString &repo_id, const QString &desc, bool enabled)
{
	if (	enabled && 
		(repo_id.contains("update", Qt::CaseInsensitive) ||
		desc.contains("update", Qt::CaseInsensitive)  ) )
	_updateRepoAvailable=true;

}

void PackageKitUpdater::slotRequireRestart(const QString& type, const QString& details)
{
	kDebug() << "RequireRestart" << type << ": " << details;

	if (type == "system")
		emit message(i18n("Reboot your system as soon as possible."));
	else if (type == "session")
		emit message(i18n("Relogin into your system as soon as possible."));
	else if (type == "application")
		emit message(i18n("Restart this application as soon as possible."));
}

void PackageKitUpdater::slotMessage(const QString& type, const QString& details)
{
	kDebug() << "message with type: " << type; 
	kDebug() << "message with details: " << details; 
	emit message(details);
}

void PackageKitUpdater::doCheckForFirmware()
{
        _get_packages_mutex->lock();
	QString dir = "/var/run/PackageKit/udev";

	kDebug() << "reading firmware directory:" + dir + " ...";
	QStringList firmwareList = QDir(dir).entryList(QDir::Files);


	for (int i=0; i < firmwareList.size(); ++i)
	{
		QFile firmwareFile ( dir + "/" + firmwareList[i]  );
		firmwareFile.open(QFile::ReadOnly);
		QString content = firmwareFile.readAll().trimmed();
		kDebug() << "found: " + firmwareFile.fileName() + " [" + content + "]" ;

		_driverSearch = true;
		pkConnector->searchFile(content);

		if (firmwareFile.isOpen())
			firmwareFile.close();

	}
        kDebug() << "done with firmware directory:" + dir + " ...";
        _get_packages_mutex->unlock();
}

void PackageKitUpdater::slotCheckDistroUpgrade()
{
	kDebug() << "asking packagekit for distro upgrades";
	pkConnector->getDistroUpgrades();
}

void PackageKitUpdater::slotDistroUpgrade( const QString &type, const QString &name, const QString &details )
{
	emit distroUpgrade( type, name, details );
}

#include "packagekit_updater.moc"

// vim: ts=2 noet sw=2
