/*
 * Copyright (C) 2007 Adrien Bustany <madcat@mymadcat.com>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#ifndef QPACKAGEKITCONNECTOR_H
#define QPACKAGEKITCONNECTOR_H

#include <QtCore>
#include <QtDBus>

#define PK_DBUS_SERVICE "org.freedesktop.PackageKit"
#define PK_DBUS_INTERFACE "org.freedesktop.PackageKit"
#define PK_DBUS_TRANSACTION_INTERFACE "org.freedesktop.PackageKit.Transaction"
#define PK_DBUS_PATH "/org/freedesktop/PackageKit"

class QPackageKitInterface;
class QPackageKitTransactionInterface;

//! Controls the PackageKit daemon
/*! This object uses a QPackageKitInterface (generated with qdbusxml2cpp) to control the PackageKit daemon.
It implements all the methods available in libpackagekit
*/
class QPackageKitConnector : public QObject
{
	Q_OBJECT
public:
	// Enums
	class Role {
	public:
		static void construct();

		//! What we asked to do
		typedef enum {
			Cancel,
			Resolve,
			Rollback,
			Get_depends,
			Get_update_detail,
			Get_description,
			Get_requires,
			Get_updates,
			Search_details,
			Search_file,
			Search_group,
			Search_name,
			Refresh_cache,
			Update_system,
			Remove_package,
			Install_package,
			Install_file,
			Update_package,
			Get_repo_list,
			Repo_enable,
			Repo_set_data,
			Unknown
		} RoleEnum;
		static QString toString(RoleEnum e);
		static RoleEnum fromString(QString s);

	private:
		static QMap<int, QString> stringValues;
	};

	class Status {
	public:
		static void construct();

		//! What we are doing
		typedef enum {
			Setup,
			Wait,
			Query,
			Remove,
			Refresh_cache,
			Download,
			Install,
			Update,
			Unknown
		} StatusEnum;
		static QString toString(StatusEnum e);
		static StatusEnum fromString(QString s);

	private:
		static QMap<int, QString> stringValues;
	};

	class Exit {
	public:
		static void construct();

		typedef enum {
			Success,
			Failed,
			Canceled,
			Unknown
		} ExitEnum;
		static QString toString(ExitEnum e);
		static ExitEnum fromString(QString s);

	private:
		static QMap<int, QString> stringValues;
	};

	class Filter {
	public:
		static void construct();

		typedef enum {
			Development,
			Installed,               
			Gui,
			Normal,          /* not development */
			Available,       /* not installed */
			Text,            /* not gui */
			Unknown
		} FilterEnum;
		static QString toString(FilterEnum e);
		static FilterEnum fromString(QString s);

	private:
		static QMap<int, QString> stringValues;
	};
	// End Enums

	//! QPackageKitConnector constructor
	QPackageKitConnector(QObject *parent = 0);

	//! Cancels the current transaction
	void Cancel();

	//! Searches for a package using its name
	bool searchName(QString name, QList<Filter::FilterEnum> filters);

	//! Orders the description of a package
	bool getDetails(QStringList packageId);
	bool getUpdateDetail(QStringList packageId);

	//! Installs a list of packages
	bool installPackages(QStringList packageIds);

	//! Installs a signature
	bool installSignature(QString type, QString key_id, QString package_id);

	//! Updates a list of packages
	bool updatePackages(QStringList packageIds);

	//! Orders a list of updates
	bool getUpdates();
	bool getDrivers();

	//! Orders a list of repos
	bool getRepoList();

	//! Refresh cache
	bool refreshCache(bool force);

	//! Accept an eule
	bool acceptEula(QString eulaId);

	//! Restart last install/update/refresh/getUpdates in case of a signature request
	bool triggerLastActionAgain();


	//! Stop the packagekit deamon 
	bool suggestDaemonQuit();
	
	//! Seach for File 
	bool searchFile(const QString& search);

        // look for distribution upgrades
        bool getDistroUpgrades();

signals:
	//! Emitted when the daemon sends us a package
	void Package(const QString &info, const QString &package_id, const QString &summary);

	//! Emitted when the current transaction is finished
	void Finished(QPackageKitConnector::Exit::ExitEnum exitCode, uint runtime);

	//! Emitted when getUpdates transaction is finished
	void GetUpdatesFinished();

	//! Emitted when updates or package installation transaction is finished
	void InstallFinished();


	//! Emitted when getRepoList transaction is finished
	void RepoListFinished();

	//! Emitted when the current transaction's progress changes 
        void ProgressChanged(uint percentage, uint subpercentage, uint elapsed, uint remaining);

	//! Emitted when we cannot track the progress of the current transaction
	void NoProgressUpdates();
	
	//! Emitted when the daemon sends us a package description
	void Details(const QString &package_id, const QString &licence, const QString &group,
			const QString &detail, const QString &url, qulonglong size); 

	//! Emitted when an error occures
	void ErrorMessage(const QString &message);

	//! Emittet when a signature is required
	void RepoSignatureRequired(const QString &package_id, const QString &repository_name,
		const QString  &key_url, const QString &key_userid, const QString &key_id,
		const QString &key_fingerprint, const QString &key_timestamp, const QString &type);

	//! Emitted when there is an eula to accept
	void EulaRequired(const QString &eula_id, const QString &package_id, const QString vendor_name,
			const QString &license_agreement);

    	void RepoDetail(const QString &repo_id, const QString &desc, bool enabled);
	void StatusChanged( const QString &status );

	void UpdateDetail(const QString &package_id, const QString &updates, const QString &obsoletes, const QString &vendor_url, 
		const QString &bugzilla_url, const QString &cve_url, const QString &restart, const QString &update_text,
                const QString &changelog, const QString &state, const QString &issued, const QString &updated);

	void RequireRestart(const QString &type, const QString &details);
	void Message(const QString &type, const QString &details);

        void DistroUpgrade( const QString &type, const QString &name, const QString &details );

public slots:

	void proxy_Package(const QString &info, const QString &package_id, const QString &summary);
	void proxy_Finished(const QString &status, uint runtime);
	void proxy_Transaction(const QString &old_tid, const QString &timespec, bool succeeded, const QString &role, uint duration, const QString &data);

        void proxy_ProgressChanged(uint percentage, uint subpercentage, uint elapsed, uint remaining);
	void proxy_Details(const QString &package_id, const QString &licence,
		const QString &group, const QString &detail, const QString &url, qulonglong size ); 
	void slotError(const QString&,const QString&);
	void proxy_EulaRequired(const QString &eula_id, const QString &package_id,
			const QString &vendor_name, const QString &license_agreement);

	void proxy_RepoSignatureRequired(const QString &package_id, const QString &repository_name,
		const QString &key_url, const QString &key_userid, const QString &key_id,
		const QString &key_fingerprint, const QString &key_timestamp, const QString &type);

	void proxy_RequireRestart(const QString &type, const QString &details);
	void proxy_Message(const QString &type, const QString &details);

    	void proxy_RepoDetail(const QString &repo_id, const QString &desc, bool enabled);

	void proxy_StatusChanged( const QString & );

 	void proxy_UpdateDetail(const QString &package_id, const QString &updates, const QString &obsoletes, const QString &vendor_url,
		 const QString &bugzilla_url, const QString &cve_url, const QString &restart, const QString &update_text,
                const QString &changelog, const QString &state, const QString &issued, const QString &updated);

        void proxy_DistroUpgrade( const QString &type, const QString &name, const QString &details );
	

private:
	// Utility functions

	// send error message 
	void sendErrorMessage(QDBusAbstractInterface*);

	//! Allocates a transaction id
	QString allocateTid(QString type);

	QString getTid();

	void disconnectFromInterface(QPackageKitTransactionInterface*);
	QPackageKitTransactionInterface* connectToInterface();

	QString _lastAction;
	QVariant _lastActionArg;

};

#endif
