#ifndef _UPDATER_H_
#define _UPDATER_H_

#include <QObject>
#include <QFlags>
#include <QPalette>
#include <QTreeWidget>
#include <kdemacros.h>

#include "constants.h"
#include "ksystemtrayicon.h"
#include "updatererror.h"

/**
 * this is the base class for updater plugins
 */
class KDE_EXPORT Updater : public QObject
{
  Q_OBJECT
    public:
  
  /**
   * Flags used to describe what can an
   * updater do.
   */
  enum Capability 
  {
    CanSelectIndividualUpdates = 0x1,
    CanProvidePackages = 0x2
  };
  Q_DECLARE_FLAGS(Capabilities, Capability)

  /**
   * constructor
   */
   Updater( QObject *parent );
   /**
    * destructor
    */
   virtual ~Updater();

   /**
    * used by gui to retrieve updater capabilities
    * Must be implemented
    */
  virtual Capabilities capabilities() const = 0;

  /**
   * called when the updater is going to quit so 
   * the backend can do cleanup
   */
  virtual void shutdown() = 0;

signals:
	// fired when an error occurred
  void appletError( const QString &desc );

	// fired when the plugin needs configuration
	// (for example by calling workflow for repo configuration)
  void pluginNeedsConfiguration( int cause );
	
	// fired after checking for updates
  void checkFinished();

	// fired after installing updates
  void installFinished();

	// fired to delevier a resolvable's description after a	slotResolvableSelected request
  void returnDescription(const QString &desc, int resolvableType);

	// fired to provide progress information
	void progress(int step, int totalSteps, QString labelText);

	// fired to show a (warning) message
  void message(const QString &);
	
	// fired if a distro upgrade is available	
  void distroUpgrade( const QString&, const QString&, const QString& );



protected slots:

  /**
  	When signaled, this slot should run whatever configuration 
  	utility this updater uses. However, this is marked as 
  	Add/Remove Servers in the GUI, so it should at least be
  	able to do that.
  **/
  virtual void slotConfigureUpdater() = 0;

  /**
  	When signaled, this slot should run the installation routine 
  	for the updater. It is important to note that you get the 
  	package list from the QListView you will _ALWAYS_ be given 
  	in the "populateUpdateList" slot. This pointer should  
  	be saved for later use in this slot (so you can fetch 
  	the package list).
  **/
  virtual void slotStartInstall() = 0;

  /**
  	When signaled, this slot should fetch a list of current 
  	updates available and add them to the listview specified. 
  **/
  virtual void slotPopulateLists(QTreeWidget *patchList, QTreeWidget *packageList, QTreeWidget *driverList) =0;
  virtual void slotCheckDrivers(QTreeWidget *patchList, QTreeWidget *packageList, QTreeWidget *driverList) = 0;
  virtual void slotCheckFirmware(QTreeWidget *patchList, QTreeWidget *packageList, QTreeWidget *driverList) = 0;

  /**
  	This slot is signaled when an update is selected and 
  	we need to put together a description for it.
  **/
  virtual void slotResolvableSelected(QTreeWidgetItem* selectedItem, int resolvableType) = 0;


  /**
	This slot is called when Packagemanagement daemon should be stopped
	for example if we want to access the packages direktly via yast
  **/

  virtual void	slotStopPackageManagementDaemon() = 0;

	
  /**
  	This slot is called when the backend should check for distro upgrades
  **/
  virtual void slotCheckDistroUpgrade() = 0;
};


// type of a resolvable
enum {
	RESOLVABLE_PATCH,							// patch 
	RESOLVABLE_PACKAGE						// package
};


// the states the applet can be in
enum {
  APPLET_NO_UPDATES,						// no updates
  APPLET_UPDATES,								// updates available available
  APPLET_CRITICAL_UPDATES,			// security updates available
  APPLET_CHECKING,	 						// checking for updates 
  APPLET_PROBLEM, 	 						// problem occured
  APPLET_INSTALLING,	 					// installing patches or packages 
  APPLET_PKGMGR_UPDATES,				// Updates available that affect the package manager
  APPLET_NO_UPDATE_REPO,	 				// no repositories are known: Start repo config workflow
  APPLET_ONLY_OPTIONAL_UPDATES,	// only optional updates are available (for trayicon) 
  APPLET_NO_NETWORK,		 				// no network connection available 
  APPLET_SYSTEM_BUSY_WHILE_CHECK,		 				// system (cpu) is busy
  APPLET_SYSTEM_BUSY_WHILE_INSTALL,		 				// system (cpu) is busy
  APPLET_SYSTEM_IS_ON_BATTERY_WHILE_CHECK,	// no power supply is connected 
  APPLET_SYSTEM_IS_ON_BATTERY_WHILE_INSTALL,	// no power supply is connected 
  APPLET_PLUGIN_PROBLEM, 	 			// problem while loading plugin
  APPLET_NEW_HARDWARE		  		// device that need driver installation has been attached
};


// the causes why a plugin need configuration
enum {
	PLUGIN_CONFIG_NO_UPDATE_REPOS	/* plugin needs configuration because no update repository is defined */
};

Q_DECLARE_OPERATORS_FOR_FLAGS(Updater::Capabilities)

#endif

