/*
   Copyright (C) 2006,2007.2008 Novell Inc

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#ifndef _ZYPP_UPDATER_H_
#define _ZYPP_UPDATER_H_

#include <qxml.h>
#include <QtCore/QList>
#include <QProcess>
#include "updater.h"

/**
	@file

	This file defines ZYpp Backend

	@author Martin Vidner <mvidner@novell.com>
	@author Duncan Mac-Vicar <dmacvicar@novell.com>
	@author Thomas Goettlicher <tgoettlicher@novell.com>
	@author Dirk Mueller <dmueller@novell.com>
*/

class LicenseDialog;

struct ZYppResolvable
{
  QString category;
  QString name;
  QString edition;
  QString source;
  QString summary;
  QString description;
  QString license;
  bool restart;					// needs system restart
  bool pkgmanager;			// affects package manager
  bool interactive;			// needs user interaction
  int	resolvableType;		// RESOLVABLE_PATCH or RESOLVABLE_PACKAGE
};

struct ZYppSource
{
  QString url;
  QString alias;
};

typedef enum
{
  Unknown,
  Update,
  UpdateDescription,
  UpdateSummary,
  UpdateLicense,
  Error,
  Warning,
  Info,
  Progress,
  Prompt,
  PromptDescription
} XmlState;

class ZYppUpdater : public Updater, public QXmlDefaultHandler
{
  Q_OBJECT
public:

  ZYppUpdater( QObject *parent, const QStringList&);
  virtual ~ZYppUpdater()
  {};

  virtual Updater::Capabilities capabilities() const;
  virtual void shutdown();

  // QXmlDefaultHandler implementation
  virtual bool startDocument();
  virtual bool endDocument();
  virtual bool startElement( const QString&, const QString&, const QString& , const QXmlAttributes& );
  virtual bool endElement( const QString&, const QString&, const QString& );
  virtual bool characters ( const QString & ch );
private slots:

  /**
  	Slot which gets called when we need to throw up the add/remove server dialog.
  */
  void slotConfigureUpdater();

  /**
  	Slot which gets called when we need to start an update. 
  	Collect our update info from the list we got passed 
  	in populateList and go. 
  */
  void slotStartInstall();

  /**
  	Slot which gets called when we need to check for updates 
  	and load up the list. 

  	@param updateList the list on the main window
  */
  void slotPopulateLists(QTreeWidget *patchList, QTreeWidget *packageList, QTreeWidget *driverList);
  void slotCheckDrivers(QTreeWidget *patchList, QTreeWidget *packageList, QTreeWidget *driverList);
  void slotCheckFirmware(QTreeWidget *patchList, QTreeWidget *packageList, QTreeWidget *driverList);

  /**
  	Slot which gets called when the user selects an update. 
  	We are expected to write a description of the update and pass it back.

  	@param item the update itself

  */
  void slotResolvableSelected(QTreeWidgetItem* item, int);

  /**
  	Slot which gets called when the user right clicks an update.

  	@param updateItem the update itself.
  	@param point where to show the menu
  */
  //void updateMenu(QTreeWidgetItem* item, const QPoint& point);

  /**
    Slot which gets called 1 minute after CheckForUpdates
    has detected "network connection being established"
   */
  //void slotCheckAgainForUpdates();
  void slotRefreshFinished( int exitCode, QProcess::ExitStatus exitStatus);
  void slotDoRefresh();
  void slotStartCheckProcess();

  void slotCheckDistroUpgrade();

protected slots:

  void slotInstSourceProcessExited();
  void slotStopPackageManagementDaemon();

  void slotShowLog();
  void slotHelperFinished(int exitCode, QProcess::ExitStatus exitStatus);
  void slotHelperStdout();
  void slotHelperStderr();

  void slotInstallProcessExited(int exitCode, QProcess::ExitStatus exitStatus);

  void slotResetXmlStream();


private:

  bool checkRegistrationRan();
  int getNetworkStatus() const;

  void doCheckForUpdates();
  bool handleMessages();

  QProcess *_process;
  QString _buffer;

  QString _stderr_buffer;
  QString _error_buffer;
  QString _message_buffer;
  QStringList _errorList;
  QStringList _warningList;
  QStringList _infoList;

  QString _install_stderr_buffer;
  QString _install_stdout_buffer;

  // parsed data, cleared between calls
  // to checkpatches
  QList<ZYppResolvable*> _resolvables;
  QList<ZYppSource> _sources;

  ZYppSource _current_source;

  XmlState _state;

  //We hold the QListView passed in "populateUpdateList" here
  QTreeWidget *tempList;
  //This holds the currently selected update in the list
  QTreeWidgetItem *currentUpdate;
  //Holds the descript for the currently selected update

  ZYppResolvable *_current_resolvable;

  int _update_counter;
  int totalUpdateCount;

  QTreeWidget *_patchList;
  QTreeWidget *_packageList;
  QTreeWidget *_driverList;
  bool _found_update_status_tag;
  bool _last_xml_parse_code;
  bool _error;
  bool _message;
  bool _noUpdateRepositories;
  bool _silent;

  LicenseDialog *licenseDialog;
  bool restartUpdater;

  // Continous xml parsing
  QXmlInputSource _xml_source;
  QXmlSimpleReader _xml_reader;
};

#endif

// vim: ts=2 noet sw=2
